<?php

declare(strict_types=1);

namespace Drupal\date_point\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemList;
use Drupal\Core\Form\FormStateInterface;
use Drupal\date_point\Data\DatePointBehavior as Behavior;
use Drupal\date_point\Data\DatePointPrecision as Precision;

/**
 * Defines an item list class for date point fields.
 *
 * @extends \Drupal\Core\Field\FieldItemList<\Drupal\date_point\Plugin\Field\FieldType\DatePointItem>
 */
final class DatePointItemList extends FieldItemList {

  /**
   * {@inheritdoc}
   */
  public function preSave(): void {
    $behavior = $this->getBehavior();
    if ($behavior === Behavior::UPDATED || ($behavior === Behavior::CREATED && $this->getEntity()->isNew())) {
      if (\count($this) === 0) {
        $this->appendItem();
      }
      foreach ($this->list as $item) {
        \assert($item instanceof DatePointItem);
        $item->setDatePoint();
      }
    }
    parent::presave();
  }

  /**
   * {@inheritdoc}
   */
  public function defaultValuesForm(array &$form, FormStateInterface $form_state): array {
    // Default values are configure through item behavior.
    return [];
  }

  /**
   * {@selfdoc}
   */
  public function getBehavior(): Behavior {
    return Behavior::from($this->getSetting('behavior'));
  }

  /**
   * {@selfdoc}
   */
  public function getPrecision(): Precision {
    return Precision::from($this->getSetting('precision'));
  }

}
