<?php

declare(strict_types=1);

namespace Drupal\date_point\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\date_point\Data\DatePointBehavior as Behavior;
use Drupal\date_point\Plugin\Field\FieldType\DatePointItemList;

/**
 * Plugin implementation of the 'date_point' widget.
 *
 * @FieldWidget(
 *   id = "date_point_date_time_local",
 *   label = @Translation("Date Time Local"),
 *   field_types = {"date_point"},
 * )
 */
final class DateTimeLocalWidget extends AbstractWidget {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    \assert($items instanceof DatePointItemList);

    $value = $items[$delta]->value;
    if ($value) {
      $value = $items[$delta]
        ->getDatePoint()
        ->setTimezone($this->getTimezone())
        ->format($items[$delta]->getFormat());
      $value = \str_replace(' ', 'T', $value);
      // datetime-local element does not support microseconds.
      $value = \substr($value, 0, 23);
    }

    $element['value'] = $element + [
      '#type' => 'date',
      '#default_value' => $value,
      '#size' => 20,
      '#attributes' => ['type' => 'datetime-local'],
      '#step' => 10 ** -$items->getPrecision()->toInteger(),
      '#disabled' => $items->getBehavior() !== Behavior::MANUAL,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state): array {
    foreach ($values as &$value) {
      $value['value'] = \str_replace('T', ' ', $value['value']);
    }
    parent::massageFormValues($values, $form, $form_state);
    return $values;
  }

}
