<?php

declare(strict_types=1);

namespace Drupal\Tests\date_point\FunctionalJavascript;

use Drupal\date_point_test\StateClockTrait;

/**
 * @group date_point
 */
final class CustomFormatterTest extends TestBase {

  use StateClockTrait;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->config('system.date')->set('timezone.default', 'Europe/Moscow')->save();
    self::getStateClock()->set(new \DateTimeImmutable('2024-01-01 15:30:56.123456'));
  }

  /**
   * {@selfdoc}
   */
  public function testFormatter(): void {
    $driver = $this->getSession()->getDriver();
    $assert_session = $this->assertSession();

    $this->createField('milliseconds');

    // Configure default formatter with default settings.
    $this->drupalGet('/admin/structure/types/manage/page/display');
    $driver->selectOption('//select[@name = "fields[field_example][type]"]', 'date_point_custom');
    $assert_session->assertWaitOnAjaxRequest();
    $xpath = <<< 'XPATH'
      //tr[@id = "field-example"]
      //div[@class = "field-plugin-summary" and text() = "Time zone: Default"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
    $xpath = <<< 'XPATH'
      //tr[@id = "field-example"]
      //div[@class = "field-plugin-summary" and text() = "Format: Y-m-d H:i:s – 2024-01-01 15:30:56"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
    $driver->click('//input[@value = "Save"]');
    $assert_session->statusMessageContains('Your settings have been saved.');

    $node_id = $this->createPage('2024-01-25 12:25:55.123');

    // Verify formatter output.
    $xpath = <<< 'XPATH'
      //div[text() = "Example"]
      /following-sibling::div
      /time[@datetime = "2024-01-25T09:25:55.123Z" and text() = "2024-01-25 12:25:55"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);

    // Change timezone and format.
    $this->drupalGet('/admin/structure/types/manage/page/display');
    $xpath = <<< 'XPATH'
      //tr[@id = "field-example"]
      //div[@class = "field-plugin-summary" and text() = "Time zone: Default"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
    $driver->click('//tr[@id = "field-example"]//input[@name = "field_example_settings_edit"]');
    $assert_session->assertWaitOnAjaxRequest();
    $driver->selectOption('//label[text() = "Time zone override"]/following-sibling::select', 'Asia/Yekaterinburg');
    $driver->setValue('//label[text() = "Date/time format"]/following-sibling::input', 'd.m.Y H:i:s.u');
    $driver->click('//input[@name = "field_example_plugin_settings_update"]');
    $assert_session->assertWaitOnAjaxRequest();
    $xpath = <<< 'XPATH'
      //tr[@id = "field-example"]
      //div[@class = "field-plugin-summary" and text() = "Time zone: Asia/Yekaterinburg"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
    $xpath = <<< 'XPATH'
      //tr[@id = "field-example"]
      //div[@class = "field-plugin-summary" and text() = "Format: d.m.Y H:i:s.u – 01.01.2024 17:30:56.123456"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
    $driver->click('//input[@value = "Save"]');

    // Verify that formatter output respects timezone override.
    $this->drupalGet('/node/' . $node_id);
    $xpath = <<< 'XPATH'
      //div[text() = "Example"]
      /following-sibling::div
      /time[@datetime = "2024-01-25T09:25:55.123Z" and text() = "25.01.2024 14:25:55.123000"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
  }

}
