<?php

declare(strict_types=1);

namespace Drupal\Tests\date_point\FunctionalJavascript;

use Drupal\date_point_test\StateClockTrait;

/**
 * @group date_point
 */
final class DefaultFormatterTest extends TestBase {

  use StateClockTrait;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->config('system.date')->set('timezone.default', 'Europe/Moscow')->save();
    self::getStateClock()->set(new \DateTimeImmutable('2024-01-01 15:30:56.123456'));
  }

  /**
   * {@selfdoc}
   */
  public function testFormatter(): void {
    $driver = $this->getSession()->getDriver();
    $assert_session = $this->assertSession();

    $this->createField('milliseconds');

    // Configure default formatter with default settings.
    $this->drupalGet('/admin/structure/types/manage/page/display');
    $xpath = <<< 'XPATH'
      //tr[@id = "field-example"]
      //div[@class = "field-plugin-summary" and text() = "Time zone: Default"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
    $xpath = <<< 'XPATH'
      //tr[@id = "field-example"]
      //div[@class = "field-plugin-summary" and text() = "Format: Default medium date – Mon, 01/01/2024 - 15:30"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
    $driver->click('//input[@value = "Save"]');
    $assert_session->statusMessageContains('Your settings have been saved.');

    $node_id = $this->createPage('2024-01-25 12:25:55.456');

    // Verify formatter output.
    $xpath = <<< 'XPATH'
      //div[text() = "Example"]
      /following-sibling::div
      /time[@datetime = "2024-01-25T09:25:55.456Z" and text() = "Thu, 01/25/2024 - 12:25"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);

    // Change timezone and format.
    $this->drupalGet('/admin/structure/types/manage/page/display');
    $xpath = <<< 'XPATH'
      //tr[@id = "field-example"]
      //div[@class = "field-plugin-summary" and text() = "Time zone: Default"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
    $driver->click('//tr[@id = "field-example"]//input[@name = "field_example_settings_edit"]');
    $assert_session->assertWaitOnAjaxRequest();
    $xpath = <<< 'XPATH'
      //label[text() = 'Format']
      /following-sibling::select
      /option[text() = "Default medium date – Mon, 01/01/2024 - 15:30"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
    $driver->selectOption('//label[text() = "Time zone override"]/following-sibling::select', 'Asia/Yekaterinburg');
    $driver->selectOption('//label[text() = "Format"]/following-sibling::select', 'rfc_3339_ms');
    $driver->click('//input[@name = "field_example_plugin_settings_update"]');
    $assert_session->assertWaitOnAjaxRequest();
    $xpath = <<< 'XPATH'
      //tr[@id = "field-example"]
      //div[@class = "field-plugin-summary" and text() = "Time zone: Asia/Yekaterinburg"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
    $xpath = <<< 'XPATH'
      //tr[@id = "field-example"]
      //div[@class = "field-plugin-summary" and text() = "Format: RFC 3339 (with milliseconds) – 2024-01-01 17:30:56.123"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
    $driver->click('//input[@value = "Save"]');

    // Verify that formatter output respects timezone override.
    $this->drupalGet('/node/' . $node_id);
    $xpath = <<< 'XPATH'
      //div[text() = "Example"]
      /following-sibling::div
      /time[@datetime = "2024-01-25T09:25:55.456Z" and text() = "2024-01-25 14:25:55.456"]
      XPATH;
    $assert_session->elementExists('xpath', $xpath);
  }

}
