<?php

declare(strict_types=1);

namespace Drupal\Tests\date_point\FunctionalJavascript;

use Drupal\date_point_test\StateClockTrait;

/**
 * @group date_point
 */
final class FieldItemTest extends TestBase {

  use StateClockTrait;

  /**
   * {@selfdoc}
   */
  public function testPrecision(): void {
    $this->createField();
    $this->configureWidget('date_point_textfield');
    $this->configureCustomFormatter();

    // Verify that only seconds were stored.
    $this->createPage('2024-01-25 12:25:55.123456');
    $this->assertDisplayedDatetime('2024-01-25 12:25:55.000000');

    // -- Verify that milliseconds were stored.
    $this->updateField('milliseconds');
    $this->createPage('2024-01-25 12:25:55.123456');
    $this->assertDisplayedDatetime('2024-01-25 12:25:55.123000');

    // -- Verify that microseconds were stored.
    $this->updateField('microseconds');
    $this->createPage('2024-01-25 12:25:55.123456');
    $this->assertDisplayedDatetime('2024-01-25 12:25:55.123456');
  }

  /**
   * {@selfdoc}
   */
  public function testBehavior(): void {
    $driver = $this->getSession()->getDriver();
    $assert_session = $this->assertSession();
    $clock = $this->getStateClock();

    $this->createField();
    $this->configureCustomFormatter();

    // -- Test 'manual' behavior.
    $this->createPage(NULL);
    $this->drupalGet('/node/1/edit');
    $driver->evaluateScript('document.querySelector(\'input[name="field_example[0][value]"]\').value = "2024-01-25T12:25:55";');
    $driver->click('//input[@value = "Save"]');
    $this->assertDisplayedDatetime('2024-01-25 12:25:55.000000');

    // -- Test 'created' behavior.
    $this->updateField(behavior: 'created');
    $clock->set(new \DateTimeImmutable('2024-01-01 15:00:00'));
    $node_id = $this->createPage(NULL);
    $this->assertDisplayedDatetime('2024-01-01 15:00:00.000000');
    $clock->set(new \DateTimeImmutable('2024-02-01 20:00:00'));
    $this->drupalGet("/node/$node_id/edit");
    $assert_session->elementExists('xpath', '//input[@name = "field_example[0][value]" and @disabled = "disabled"]');
    $driver->click('//input[@value = "Save"]');
    // Check that the value has not been updated.
    $this->assertDisplayedDatetime('2024-01-01 15:00:00.000000');

    // -- Test 'updated' behavior.
    $this->updateField(behavior: 'updated');
    $clock->set(new \DateTimeImmutable('2024-01-01 15:00:00'));
    $node_id = $this->createPage(NULL);
    $clock->set(new \DateTimeImmutable('2024-02-01 20:00:00'));
    $this->drupalGet("/node/$node_id/edit");
    $assert_session->elementExists('xpath', '//input[@name = "field_example[0][value]" and @disabled = "disabled"]');
    $driver->click('//input[@value = "Save"]');
    // Check that the value has been updated.
    $this->assertDisplayedDatetime('2024-02-01 20:00:00.000000');
  }

}
