<?php

declare(strict_types=1);

namespace Drupal\Tests\date_point\FunctionalJavascript;

/**
 * @group date_point
 */
final class PlainFormatterTest extends TestBase {

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->config('system.date')->set('timezone.default', 'Europe/Moscow')->save();
  }

  /**
   * {@selfdoc}
   */
  public function testFormatter(): void {
    $driver = $this->getSession()->getDriver();
    $assert_session = $this->assertSession();

    $this->createField('milliseconds');

    // Configure plain formatter with default settings.
    $this->drupalGet('/admin/structure/types/manage/page/display');
    $driver->selectOption('//select[@name = "fields[field_example][type]"]', 'date_point_plain');
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->elementExists('xpath', '//tr[@id = "field-example"]//div[@class = "field-plugin-summary" and text() = "Time zone: Default"]');
    $driver->click('//input[@value = "Save"]');
    $assert_session->statusMessageContains('Your settings have been saved.');

    $node_id = $this->createPage('2024-01-25 12:25:55.456');

    // Verify formatter output.
    $assert_session->elementExists('xpath', '//div[text() = "Example"]/following-sibling::div[text() = "2024-01-25 12:25:55.456"]');

    // Change formatter timezone.
    $this->drupalGet('/admin/structure/types/manage/page/display');
    $assert_session->elementExists('xpath', '//tr[@id = "field-example"]//div[@class = "field-plugin-summary" and text() = "Time zone: Default"]');
    $driver->click('//tr[@id = "field-example"]//input[@name = "field_example_settings_edit"]');
    $assert_session->assertWaitOnAjaxRequest();
    $driver->selectOption('//label[text() = "Time zone override"]/following-sibling::select', 'Asia/Yekaterinburg');
    $driver->click('//input[@name = "field_example_plugin_settings_update"]');
    $assert_session->assertWaitOnAjaxRequest();
    $assert_session->elementExists('xpath', '//tr[@id = "field-example"]//div[@class = "field-plugin-summary" and text() = "Time zone: Asia/Yekaterinburg"]');
    $driver->click('//input[@value = "Save"]');

    // Verify that formatter output respects timezone override.
    $this->drupalGet('/node/' . $node_id);
    $assert_session->elementExists('xpath', '//div[text() = "Example"]/following-sibling::div[text() = "2024-01-25 14:25:55.456"]');
  }

}
